package com.speedingplanet.banking.resources;

import com.google.common.base.Optional;
import com.speedingplanet.banking.api.Transaction;
import com.speedingplanet.banking.db.TransactionDAO;
import io.dropwizard.hibernate.UnitOfWork;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.*;
import java.net.URI;
import java.util.List;

@Path("/tx")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class TransactionResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(TransactionResource.class);
  private final TransactionDAO txDAO;

  @Context
  UriInfo uriInfo;

  public TransactionResource(TransactionDAO txDAO) {
    this.txDAO = txDAO;
  }

  @GET
  @UnitOfWork
  public List<Transaction> getTransactions(@Context UriInfo uriInfo) {
    MultivaluedMap<String, String> queryParams = uriInfo.getQueryParameters();

    if (queryParams.size() == 0) {
      return txDAO.findAll();
    } else {
      return txDAO.query(queryParams);
    }
  }

  @Path("/{txId:[0-9]+}")
  @GET
  @UnitOfWork
  public Transaction getTransaction(@PathParam("txId") Integer txId) {
    Optional<Transaction> tx = Optional.fromNullable(txDAO.findById(txId));
    if (!tx.isPresent()) {
      throw new NotFoundException(String.format("Transaction id %d not found.", txId));
    } else {
      return tx.get();
    }
  }

  @Path("/payee")
  @GET
  @UnitOfWork
  public List<Transaction> getTransactionsByPayeeName(@DefaultValue("") @MatrixParam("name") String payeeName) {
    List<Transaction> results;
    if (payeeName.isEmpty()) {
      results = txDAO.findAll();
    } else {
      results = txDAO.findByPayeeName(payeeName);
    }

    if (results.size() == 0) {
      throw new NotFoundException("Could not find any transactions with that Payee.");
    }

    return results;
  }

  @POST
  @UnitOfWork
  public Response insertTransaction(Transaction tx) {
    if (tx.getTxId() != 0) {
      LOGGER.warn("Attempted to insert transaction with a txId, rejected.");

      // 422 is an Unprocessable entity
      throw new WebApplicationException("Attempted to insert transaction with a txId, rejected.",
          422);
    }

    Transaction responseTx = txDAO.create(tx);
    URI uri = uriInfo
        .getBaseUriBuilder()
        .path(TransactionResource.class)
        .path("" + responseTx.getTxId())
        .build();
    return Response.created(uri).entity(responseTx).build();
  }

  @Path("/{txId:[0-9]+}")
  @POST
  @UnitOfWork
  public Response updateTransaction(@PathParam("txId") int txId, Transaction tx) {
    if (txId == 0) {
      throw new WebApplicationException("Attempted to update a transaction without a txId, rejected.",
          Response.Status.BAD_REQUEST);
    }

    return Response.ok(txDAO.update(tx)).build();
  }

  @Path("/{txId:[0-9]+}")
  @PUT
  @UnitOfWork
  public Response putUpdateTransaction(@PathParam("txId") int txId, Transaction tx) {
    return updateTransaction(txId, tx);
  }

}
