package com.speedingplanet.equities.resources;

import com.google.common.base.Optional;
import com.speedingplanet.equities.api.Order;
import com.speedingplanet.equities.db.OrderDAO;
import io.dropwizard.hibernate.UnitOfWork;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.net.URI;
import java.util.List;

@Path("/order")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class OrderResource {
  private static final Logger LOGGER = LoggerFactory.getLogger(OrderResource.class);
  private final OrderDAO dao;

  @Context
  UriInfo uriInfo;

  public OrderResource(OrderDAO dao) {
    this.dao = dao;
  }

  @GET
  @UnitOfWork
  public List<Order> getOrders() {
    return dao.findAll();
  }

  @Path("/{orderId:[0-9]+}")
  @GET
  @UnitOfWork
  public Order getOrder(@PathParam("orderId") int id) {
    Optional<Order> order = Optional.fromNullable(dao.findById(id));

    if (!order.isPresent()) {
      throw new NotFoundException(String.format("Order for orderId %d not found.", id));
    }

    return order.get();
  }

  @POST
  @UnitOfWork
  public Response insertOrder(@Valid Order o) {
    if (o.getOrderId() != 0) {
      LOGGER.warn("Attempted to insert Order with a orderId, rejected.");

      // 422 is an Unprocessable entity
      throw new WebApplicationException("Attempted to insert Order with a orderId, rejected.",
          422);
    }

    Order responseTx = dao.create(o);
    URI uri = uriInfo
        .getBaseUriBuilder()
        .path(OrderResource.class)
        .path("" + responseTx.getOrderId())
        .build();
    return Response.created(uri).entity(responseTx).build();
  }

  @Path("/{orderId:[0-9]+}")
  @POST
  @UnitOfWork
  public Response updateOrder(Order o) {
    if (o.getOrderId() == 0) {
      throw new WebApplicationException("Attempted to update a Order without a orderId, rejected.",
          Response.Status.CONFLICT);
    }

    return Response.ok(dao.update(o)).build();
  }

  @Path("/{orderId:[0-9]+}")
  @PUT
  @UnitOfWork
  public Response putUpdateOrder(Order o) {
    return updateOrder(o);
  }
}
