package com.speedingplanet.equities.resources;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.speedingplanet.equities.api.Position;
import com.speedingplanet.equities.api.Security;
import com.speedingplanet.equities.db.PositionDAO;
import io.dropwizard.jackson.Jackson;
import io.dropwizard.testing.junit.ResourceTestRule;
import org.joda.time.DateTime;
import org.junit.After;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.runners.MockitoJUnitRunner;

import javax.ws.rs.client.Entity;
import javax.ws.rs.core.*;
import java.io.IOException;
import java.util.List;

import static io.dropwizard.testing.FixtureHelpers.fixture;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class PositionResourceTest {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private static final PositionDAO dao = mock(PositionDAO.class);

  @ClassRule
  public static final ResourceTestRule resources = ResourceTestRule.builder()
      .addResource(new PositionResource(dao))
      .build();

  @Captor
  ArgumentCaptor<Position> posCaptor;

  @After
  public void tearDown() {
    reset(dao);
  }

  @Test
  public void testGetPosition() {
    final Position pos = new Position(1000, "AAPL", 100, 100.0, new DateTime("2015-01-04T00:00:00.000"));

    when(dao.findById(1000)).thenReturn(pos);

    Position returnedPos = resources.client().target("/position/1000").request().get(Position.class);
    assertThat(returnedPos).isEqualTo(pos);
    verify(dao).findById(1000);

  }

  @Test
  public void testGetPositions() throws IOException {
    // Mocked data
    CollectionType javaType = MAPPER.getTypeFactory().constructCollectionType(List.class, Position.class);
    List<Position> positions = MAPPER.readValue(fixture("fixtures/query-positions.json"), javaType);
    MultivaluedMap<String, String> queryParams = new MultivaluedHashMap<String, String>();

    queryParams.add("symbol", "AAPL");

    when(dao.query(queryParams)).thenReturn(positions);

    assertThat(resources.client().target("/position?symbol=AAPL").request().get(new GenericType<List<Position>>() {
    }))
        .isEqualTo(positions);
    verify(dao).query(queryParams);
  }

  @Test
  public void testInsertPosition() throws IOException {
    Position insertPos = MAPPER.readValue(fixture("fixtures/insert-position.json"), Position.class);

    when(dao.create(any(Position.class))).thenReturn(insertPos);

    Response response = resources
        .client()
        .target("/position")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .post(Entity.entity(insertPos, MediaType.APPLICATION_JSON_TYPE));

    assertThat(response.getStatusInfo()).isEqualTo(Response.Status.CREATED);
    verify(dao).create(posCaptor.capture());
    assertThat(posCaptor.getValue()).isEqualTo(insertPos);

  }

  @Test
  public void testUpdateTransaction() throws IOException{
    Position updatePos = MAPPER.readValue(fixture("fixtures/update-position.json"), Position.class);

    when(dao.update(any(Position.class))).thenReturn(updatePos);

    Response response = resources
        .client()
        .target("/position/100")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .post(Entity.entity(updatePos, MediaType.APPLICATION_JSON_TYPE));

    assertThat(response.getStatusInfo()).isEqualTo(Response.Status.OK);
    verify(dao).update(posCaptor.capture());
    assertThat(posCaptor.getValue()).isEqualTo(updatePos);
  }
}
